@@ -19,6 +19,7 @@
 
 package org.apache.cxf.endpoint;
 
+import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
@@ -46,6 +47,7 @@ import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.interceptor.AbstractBasicInterceptorProvider;
 import org.apache.cxf.interceptor.ClientOutFaultObserver;
+import org.apache.cxf.interceptor.Fault;
 import org.apache.cxf.interceptor.Interceptor;
 import org.apache.cxf.interceptor.InterceptorChain;
 import org.apache.cxf.interceptor.InterceptorProvider;
@@ -68,6 +70,7 @@ import org.apache.cxf.service.model.OperationInfo;
 import org.apache.cxf.service.model.ServiceInfo;
 import org.apache.cxf.transport.Conduit;
 import org.apache.cxf.transport.MessageObserver;
+import org.apache.cxf.workqueue.SynchronousExecutor;
 import org.apache.cxf.wsdl11.WSDLServiceFactory;
 
 public class ClientImpl
@@ -301,7 +304,7 @@ public class ClientImpl
                            Object[] params,
                            Exchange exchange) throws Exception {
         Map<String, Object> context = new HashMap<String, Object>();
-        Map<String, Object> resp = getResponseContext();
+        Map<String, Object> resp = new HashMap<String, Object>();
         resp.clear();
         Map<String, Object> req = new HashMap<String, Object>(getRequestContext());
         context.put(RESPONSE_CONTEXT, resp);
@@ -375,18 +378,47 @@ public class ClientImpl
     public void invoke(ClientCallback callback,
                        BindingOperationInfo oi,
                        Object... params) throws Exception {
+        invoke(callback, oi, params, null, null);
+    }
+    
+    public void invoke(ClientCallback callback,
+                       BindingOperationInfo oi,
+                       Object[] params,
+                       Map<String, Object> context) throws Exception {
+        invoke(callback, oi, params, context, null);
+    }
+    
+    public void invoke(ClientCallback callback,
+                       BindingOperationInfo oi,
+                       Object[] params,
+                       Exchange exchange) throws Exception {
+        invoke(callback, oi, params, null, exchange);
+    }
+    
+    @SuppressWarnings("unchecked")
+    public void invoke(ClientCallback callback,
+                       BindingOperationInfo oi,
+                       Object[] params,
+                       Map<String, Object> context,
+                       Exchange exchange) throws Exception {
         Bus origBus = BusFactory.getThreadDefaultBus(false);
         BusFactory.setThreadDefaultBus(bus);
         try {
-            Exchange exchange = new ExchangeImpl();
+            if (exchange == null) {
+                exchange = new ExchangeImpl();
+            }
             exchange.setSynchronous(false);
             Endpoint endpoint = getEndpoint();
-            Map<String, Object> context = new HashMap<String, Object>();
-            Map<String, Object> resp = getResponseContext();
-            resp.clear();
-            Map<String, Object> reqContext = new HashMap<String, Object>(getRequestContext());
-            context.put(RESPONSE_CONTEXT, resp);
-            context.put(REQUEST_CONTEXT, reqContext);
+            if (context == null) {
+                context = new HashMap<String, Object>();
+                Map<String, Object> resp = new HashMap<String, Object>();
+                resp.clear();
+                Map<String, Object> reqContext = new HashMap<String, Object>(getRequestContext());
+                context.put(RESPONSE_CONTEXT, resp);
+                context.put(REQUEST_CONTEXT, reqContext);
+            }
+                        
+            Map<String, Object> reqContext = (Map<String, Object>)context.get(REQUEST_CONTEXT);
 
             Message message = endpoint.getBinding().createMessage();
             message.put(Message.INVOCATION_CONTEXT, context);
@@ -479,9 +511,34 @@ public class ClientImpl
 
             // add additional interceptors and such
             modifyChain(chain, message, false);
-            
-            // execute chain
-            chain.doIntercept(message);
+            try {
+                // execute chain
+                chain.doIntercept(message);
+            } catch (Fault fault) {
+                if (fault.getCause().getCause() instanceof IOException
+                        || fault.getCause() instanceof IOException) {
+                    String soap11NS = "http://schemas.xmlsoap.org/soap/envelope/";
+                    String soap12NS = "http://www.w3.org/2003/05/soap-envelope";
+                    QName faultCode = fault.getFaultCode();
+                    //for SoapFault, if it's underlying cause is IOException, 
+                    //it means something like server is down or can't create 
+                    //connection, according to soap spec we should set fault as
+                    //Server Fault
+                    if (faultCode.getNamespaceURI().equals(
+                            soap11NS)
+                            && faultCode.getLocalPart().equals("Client")) {
+                        faultCode = new QName(soap11NS, "Server");
+                        fault.setFaultCode(faultCode);
+                    }
+                    if (faultCode.getNamespaceURI().equals(
+                            soap12NS)
+                            && faultCode.getLocalPart().equals("Sender")) {
+                        faultCode = new QName(soap12NS, "Receiver");
+                        fault.setFaultCode(faultCode);
+                    }
+                }
+                throw fault;
+            }
 
             return processResult(message, exchange, oi, resContext);
 
@@ -494,8 +551,11 @@ public class ClientImpl
                                    Exchange exchange,
                                    BindingOperationInfo oi,
                                    Map<String, Object> resContext) throws Exception {
-     // Check to see if there is a Fault from the outgoing chain
-        Exception ex = message.getContent(Exception.class);
+        Exception ex = null;
+        // Check to see if there is a Fault from the outgoing chain if it's an out Message
+        if (!message.get(Message.INBOUND_MESSAGE).equals(Boolean.TRUE)) {
+            ex = message.getContent(Exception.class);
+        }
         boolean mepCompleteCalled = false;
         if (ex != null) {
             getConduitSelector().complete(exchange);
@@ -532,8 +592,9 @@ public class ClientImpl
             if (null != resContext) {
                 resContext.putAll(inMsg);
                 if (LOG.isLoggable(Level.FINE)) {
-                    LOG.fine("set responseContext to be" + responseContext);
+                    LOG.fine("set responseContext to be" + resContext);
                 }
+                responseContext.put(Thread.currentThread(), resContext);
             }
             resList = inMsg.getContent(List.class);
         }
@@ -596,7 +657,7 @@ public class ClientImpl
 
     public void onMessage(Message message) {
 
-        Endpoint endpoint = message.getExchange().get(Endpoint.class);
+        Endpoint endpoint = message.getExchange().getEndpoint();
         if (endpoint == null) {
             // in this case correlation will occur outside the transport,
             // however there's a possibility that the endpoint may have been
@@ -611,19 +672,19 @@ public class ClientImpl
         message.put(Message.INBOUND_MESSAGE, Boolean.TRUE);
         PhaseManager pm = bus.getExtension(PhaseManager.class);
 
-        List<Interceptor> i1 = bus.getInInterceptors();
+        List<Interceptor<? extends Message>> i1 = bus.getInInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by bus: " + i1);
         }
-        List<Interceptor> i2 = getInInterceptors();
+        List<Interceptor<? extends Message>> i2 = getInInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by client: " + i2);
         }
-        List<Interceptor> i3 = endpoint.getInInterceptors();
+        List<Interceptor<? extends Message>> i3 = endpoint.getInInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
-            LOG.fine("Interceptors contributed by endpoint: " + i2);
+            LOG.fine("Interceptors contributed by endpoint: " + i3);
         }
-        List<Interceptor> i4 = endpoint.getBinding().getInInterceptors();
+        List<Interceptor<? extends Message>> i4 = endpoint.getBinding().getInInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by binding: " + i4);
         }
@@ -669,7 +730,31 @@ public class ClientImpl
             } else if (message.getContent(Exception.class) != null) {
                 outFaultObserver.onMessage(message);
             } else {
-                chain.doIntercept(message);
+                callback = message.getExchange().get(ClientCallback.class);
+
+                if (callback != null && !isPartialResponse(message)) {
+                    try {
+                        chain.doIntercept(message);
+                    } catch (Throwable error) {
+                        //so that asyn callback handler get chance to 
+                        //handle non-runtime exceptions
+                        message.getExchange().setInMessage(message);
+                        Map<String, Object> resCtx = CastUtils
+                                .cast((Map<?, ?>) message.getExchange()
+                                        .getOutMessage().get(
+                                                Message.INVOCATION_CONTEXT));
+                        resCtx = CastUtils.cast((Map<?, ?>) resCtx
+                                .get(RESPONSE_CONTEXT));
+                        if (resCtx != null) {
+                            responseContext.put(Thread.currentThread(), resCtx);
+                        }
+                        callback.handleException(resCtx, error);
+
+                    }
+                } else {
+                    chain.doIntercept(message);
+                }
+                 
             }
 
             callback = message.getExchange().get(ClientCallback.class);
@@ -681,7 +766,9 @@ public class ClientImpl
                                                                 .getOutMessage()
                                                                 .get(Message.INVOCATION_CONTEXT));
                 resCtx = CastUtils.cast((Map<?, ?>)resCtx.get(RESPONSE_CONTEXT));
-
+                if (resCtx != null) {
+                    responseContext.put(Thread.currentThread(), resCtx);
+                }
                 try {
                     Object obj[] = processResult(message, message.getExchange(),
                                                  null, resCtx);
@@ -746,13 +833,20 @@ public class ClientImpl
         if (exchange.isSynchronous() || executor == null) {
             exchange.put(MessageObserver.class, this);
         } else {
+            exchange.put(Executor.class, executor);
             exchange.put(MessageObserver.class, new MessageObserver() {
                 public void onMessage(final Message message) {
-                    executor.execute(new Runnable() {
-                        public void run() {
-                            ClientImpl.this.onMessage(message);
-                        }
-                    });
+                    if (!message.getExchange()
+                        .containsKey(Executor.class.getName() + ".USING_SPECIFIED")) {
+                        
+                        executor.execute(new Runnable() {
+                            public void run() {
+                                ClientImpl.this.onMessage(message);
+                            }
+                        });
+                    } else {
+                        ClientImpl.this.onMessage(message);
+                    }
                 }
             });
         }
@@ -791,23 +885,23 @@ public class ClientImpl
 
         PhaseManager pm = bus.getExtension(PhaseManager.class);
 
-        List<Interceptor> i1 = bus.getOutInterceptors();
+        List<Interceptor<? extends Message>> i1 = bus.getOutInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by bus: " + i1);
         }
-        List<Interceptor> i2 = getOutInterceptors();
+        List<Interceptor<? extends Message>> i2 = getOutInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by client: " + i2);
         }
-        List<Interceptor> i3 = endpoint.getOutInterceptors();
+        List<Interceptor<? extends Message>> i3 = endpoint.getOutInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by endpoint: " + i3);
         }
-        List<Interceptor> i4 = endpoint.getBinding().getOutInterceptors();
+        List<Interceptor<? extends Message>> i4 = endpoint.getBinding().getOutInterceptors();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.fine("Interceptors contributed by binding: " + i4);
         }
-        List<Interceptor> i5 = null;
+        List<Interceptor<? extends Message>> i5 = null;
         if (endpoint.getService().getDataBinding() instanceof InterceptorProvider) {
             i5 = ((InterceptorProvider)endpoint.getService().getDataBinding()).getOutInterceptors();
             if (LOG.isLoggable(Level.FINE)) {
@@ -836,7 +930,8 @@ public class ClientImpl
             }
         }
         String key = in ? Message.IN_INTERCEPTORS : Message.OUT_INTERCEPTORS;
-        Collection<Interceptor> is = CastUtils.cast((Collection<?>)ctx.get(key));
+        Collection<Interceptor<? extends Message>> is 
+            = CastUtils.cast((Collection<?>)ctx.get(key));
         if (is != null) {
             chain.add(is);
         }
@@ -858,19 +953,17 @@ public class ClientImpl
         return getConduitSelector(null);
     }
 
-    protected final synchronized ConduitSelector getConduitSelector(
+    protected final ConduitSelector getConduitSelector(
         ConduitSelector override
     ) {
         if (null == conduitSelector) {
-            setConduitSelector(override != null
-                               ? override
-                               : new UpfrontConduitSelector());
+            setConduitSelector(override);
         }
         return conduitSelector;
     }
 
-    public final void setConduitSelector(ConduitSelector selector) {
-        conduitSelector = selector;
+    public final synchronized void setConduitSelector(ConduitSelector selector) {
+        conduitSelector = selector == null ? new UpfrontConduitSelector() : selector;
     }
 
     private boolean isPartialResponse(Message in) {
@@ -911,7 +1004,9 @@ public class ClientImpl
 
 
     public void setExecutor(Executor executor) {
-        this.executor = executor;
+        if (!SynchronousExecutor.isA(executor)) {
+            this.executor = executor;
+        }
     }
 
 
