@@ -27,14 +27,16 @@ import java.net.HttpURLConnection;
 import java.net.URI;
 import java.net.URL;
 import java.text.SimpleDateFormat;
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.logging.Logger;
 
-import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Cookie;
 import javax.ws.rs.core.EntityTag;
 import javax.ws.rs.core.HttpHeaders;
@@ -57,6 +59,7 @@ import org.apache.cxf.interceptor.Interceptor;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.impl.UriBuilderImpl;
 import org.apache.cxf.jaxrs.model.ParameterType;
+import org.apache.cxf.jaxrs.model.URITemplate;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.jaxrs.utils.HttpUtils;
 import org.apache.cxf.jaxrs.utils.InjectionUtils;
@@ -83,16 +86,14 @@ public class AbstractClient implements Client {
     private static final String RESPONSE_CONTEXT = "ResponseContext";
     
     protected ClientConfiguration cfg = new ClientConfiguration();
+    private ClientState state;
     
-    private MultivaluedMap<String, String> requestHeaders = new MetadataMap<String, String>();
-    private ResponseBuilder responseBuilder;
+    protected AbstractClient(URI baseURI) {
+        this.state = new LocalClientState(baseURI);
+    }
     
-    private URI baseURI;
-    private UriBuilder currentBuilder;
-
-    protected AbstractClient(URI baseURI, URI currentURI) {
-        this.baseURI = baseURI;
-        this.currentBuilder = new UriBuilderImpl(currentURI);
+    protected AbstractClient(ClientState initialState) {
+        this.state = initialState;
     }
     
     /**
@@ -102,20 +103,25 @@ public class AbstractClient implements Client {
         if (values == null) {
             throw new IllegalArgumentException();
         }
-        if (HttpHeaders.CONTENT_TYPE.equals(name) && values.length > 1) {
-            throw new WebApplicationException();
-        }
-        for (Object o : values) {
-            requestHeaders.add(name, o.toString());
+        if (HttpHeaders.CONTENT_TYPE.equals(name)) {
+            if (values.length > 1) {
+                throw new IllegalArgumentException("Content-Type can have a single value only");
+            }
+            type(values[0].toString());
+        } else {
+            for (Object o : values) {
+                possiblyAddHeader(name, o.toString());
+            }
         }
         return this;
     }
 
+    
     /**
      * {@inheritDoc}
      */
     public Client headers(MultivaluedMap<String, String> map) {
-        requestHeaders.putAll(map);
+        state.getRequestHeaders().putAll(map);
         return this;
     }
     
@@ -124,7 +130,7 @@ public class AbstractClient implements Client {
      */
     public Client accept(MediaType... types) {
         for (MediaType mt : types) {
-            requestHeaders.add(HttpHeaders.ACCEPT, mt.toString());
+            possiblyAddHeader(HttpHeaders.ACCEPT, mt.toString());
         }
         return this;
     }
@@ -140,7 +146,7 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public Client type(String type) {
-        requestHeaders.putSingle(HttpHeaders.CONTENT_TYPE, type);
+        state.getRequestHeaders().putSingle(HttpHeaders.CONTENT_TYPE, type);
         return this;
     }
 
@@ -149,7 +155,7 @@ public class AbstractClient implements Client {
      */
     public Client accept(String... types) {
         for (String type : types) {
-            requestHeaders.add(HttpHeaders.ACCEPT, type);
+            possiblyAddHeader(HttpHeaders.ACCEPT, type);
         }
         return this;
     }
@@ -158,7 +164,7 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public Client cookie(Cookie cookie) {
-        requestHeaders.add(HttpHeaders.COOKIE, cookie.toString());
+        possiblyAddHeader(HttpHeaders.COOKIE, cookie.toString());
         return this;
     }
 
@@ -168,7 +174,7 @@ public class AbstractClient implements Client {
     public Client modified(Date date, boolean ifNot) {
         SimpleDateFormat dateFormat = HttpUtils.getHttpDateFormat();
         String hName = ifNot ? HttpHeaders.IF_UNMODIFIED_SINCE : HttpHeaders.IF_MODIFIED_SINCE;
-        requestHeaders.putSingle(hName, dateFormat.format(date));
+        state.getRequestHeaders().putSingle(hName, dateFormat.format(date));
         return this;
     }
 
@@ -176,7 +182,7 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public Client language(String language) {
-        requestHeaders.putSingle(HttpHeaders.CONTENT_LANGUAGE, language);
+        state.getRequestHeaders().putSingle(HttpHeaders.CONTENT_LANGUAGE, language);
         return this;
     }
 
@@ -185,7 +191,7 @@ public class AbstractClient implements Client {
      */
     public Client match(EntityTag tag, boolean ifNot) {
         String hName = ifNot ? HttpHeaders.IF_NONE_MATCH : HttpHeaders.IF_MATCH; 
-        requestHeaders.putSingle(hName, tag.toString());
+        state.getRequestHeaders().putSingle(hName, tag.toString());
         return this;
     }
 
@@ -194,7 +200,7 @@ public class AbstractClient implements Client {
      */
     public Client acceptLanguage(String... languages) {
         for (String s : languages) {
-            requestHeaders.add(HttpHeaders.ACCEPT_LANGUAGE, s);
+            possiblyAddHeader(HttpHeaders.ACCEPT_LANGUAGE, s);
         }
         return this;
     }
@@ -204,7 +210,7 @@ public class AbstractClient implements Client {
      */
     public Client acceptEncoding(String... encs) {
         for (String s : encs) {
-            requestHeaders.add(HttpHeaders.ACCEPT_ENCODING, s);
+            possiblyAddHeader(HttpHeaders.ACCEPT_ENCODING, s);
         }
         return this;
     }
@@ -213,7 +219,7 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public Client encoding(String enc) {
-        requestHeaders.putSingle(HttpHeaders.CONTENT_ENCODING, enc);
+        state.getRequestHeaders().putSingle(HttpHeaders.CONTENT_ENCODING, enc);
         return this;
     }
 
@@ -222,7 +228,7 @@ public class AbstractClient implements Client {
      */
     public MultivaluedMap<String, String> getHeaders() {
         MultivaluedMap<String, String> map = new MetadataMap<String, String>();
-        map.putAll(requestHeaders);
+        map.putAll(state.getRequestHeaders());
         return map;
     }
     
@@ -230,7 +236,7 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public URI getBaseURI() {
-        return baseURI;
+        return state.getBaseURI();
     }
     
     /**
@@ -244,43 +250,72 @@ public class AbstractClient implements Client {
      * {@inheritDoc}
      */
     public Response getResponse() {
-        if (responseBuilder == null) {
+        if (state.getResponseBuilder() == null) {
             return null;
         }
-        return responseBuilder.build();
+        return state.getResponseBuilder().build();
     }
     
     /**
      * {@inheritDoc}
      */
     public Client reset() {
-        requestHeaders.clear();
-        resetResponse();
+        state.reset();
         return this;
     }
 
+    private void possiblyAddHeader(String name, String value) {
+        if (!isDuplicate(name, value)) {
+            state.getRequestHeaders().add(name, value);
+        }
+    }
+    
+    private boolean isDuplicate(String name, String value) {
+        List<String> values = state.getRequestHeaders().get(name);
+        return values != null && values.contains(value) ? true : false;
+    }
+    
+    protected ClientState getState() {
+        return state;
+    }
     
     protected UriBuilder getCurrentBuilder() {
-        return currentBuilder;
+        return state.getCurrentBuilder();
     }
 
     protected void resetResponse() {
-        responseBuilder = null;
+        state.setResponseBuilder(null);
     }
     
     protected void resetBaseAddress(URI uri) {
-        baseURI = uri;
+        state.setBaseURI(uri);
         resetCurrentBuilder(uri);
     }
     
     protected void resetCurrentBuilder(URI uri) {
-        currentBuilder = new UriBuilderImpl(uri);
+        state.setCurrentBuilder(new UriBuilderImpl(uri));
+    }
+    
+    protected MultivaluedMap<String, String> getTemplateParametersMap(URITemplate template, 
+                                                                      List<Object> values) {
+        if (values != null && values.size() != 0) { 
+            List<String> vars = template.getVariables();
+            MultivaluedMap<String, String> templatesMap =  new MetadataMap<String, String>(vars.size());
+            for (int i = 0; i < vars.size(); i++) {
+                if (i < values.size()) {
+                    templatesMap.add(vars.get(i), values.get(i).toString());
+                }
+            }
+            return templatesMap;
+        } 
+        return null;
     }
     
     protected ResponseBuilder setResponseBuilder(HttpURLConnection conn, Exchange exchange) throws Throwable {
         Message inMessage = exchange.getInMessage();
         if (conn == null) {
-            throw new WebApplicationException(); 
+            // unlikely to occur
+            throw new ClientWebApplicationException("HTTP Connection is null"); 
         }
         Integer responseCode = (Integer)exchange.get(Message.RESPONSE_CODE);
         if (responseCode == null) {
@@ -297,19 +332,26 @@ public class AbstractClient implements Client {
             }
         } 
         int status = responseCode.intValue();
-        responseBuilder = Response.status(status);
+        ResponseBuilder currentResponseBuilder = Response.status(status);
+        
         for (Map.Entry<String, List<String>> entry : conn.getHeaderFields().entrySet()) {
             if (null == entry.getKey()) {
                 continue;
             }
-            if (HttpUtils.isDateRelatedHeader(entry.getKey())) {
-                responseBuilder.header(entry.getKey(), entry.getValue());
-            } else if (entry.getValue().size() > 0) {
-                String[] values = entry.getValue().get(0).split(",");
-                for (String s : values) {
-                    String theValue = s.trim();
-                    if (theValue.length() > 0) {
-                        responseBuilder.header(entry.getKey(), theValue);
+            if (entry.getValue().size() > 0) {
+                if (HttpUtils.isDateRelatedHeader(entry.getKey())) {
+                    currentResponseBuilder.header(entry.getKey(), entry.getValue().get(0));
+                    continue;                    
+                }
+                for (String val : entry.getValue()) {
+                    boolean splitPossible = !(HttpHeaders.SET_COOKIE.equalsIgnoreCase(entry.getKey())
+                        && val.contains(HttpHeaders.EXPIRES));
+                    String[] values = splitPossible ? val.split(",") : new String[]{val};
+                    for (String s : values) {
+                        String theValue = s.trim();
+                        if (theValue.length() > 0) {
+                            currentResponseBuilder.header(entry.getKey(), theValue);
+                        }
                     }
                 }
             }
@@ -321,19 +363,21 @@ public class AbstractClient implements Client {
         if (status >= 400) {
             try {
                 InputStream errorStream = mStream == null ? conn.getErrorStream() : mStream;
-                responseBuilder.entity(errorStream);
+                currentResponseBuilder.entity(errorStream);
             } catch (Exception ex) {
                 // nothing we can do really
             }
         } else {
             try {
                 InputStream stream = mStream == null ? conn.getInputStream() : mStream;
-                responseBuilder.entity(stream);
+                currentResponseBuilder.entity(stream);
             } catch (Exception ex) {
                 // it may that the successful response has no response body
             }
         }
-        return responseBuilder;
+        ResponseBuilder rb = currentResponseBuilder.clone();
+        state.setResponseBuilder(currentResponseBuilder);
+        return rb;
     }
 
     @SuppressWarnings("unchecked")
@@ -355,13 +399,14 @@ public class AbstractClient implements Client {
         if (mbw != null) {
             try {
                 mbw.writeTo(o, cls, type, anns, contentType, headers, os);
-                os.flush();
+                if (os != null) {
+                    os.flush();
+                }
             } catch (Exception ex) {
-                throw new WebApplicationException(ex);
+                reportMessageHandlerProblem("MSG_WRITER_PROBLEM", cls, contentType, ex, null);
             }
-             
         } else {
-            reportNoMessageHandler("NO_MSG_WRITER", cls, contentType);
+            reportMessageHandlerProblem("NO_MSG_WRITER", cls, contentType, null, null);
         }
                                                                                  
     }
@@ -380,7 +425,7 @@ public class AbstractClient implements Client {
                 Object length = r.getMetadata().getFirst(HttpHeaders.CONTENT_LENGTH);
                 if (length == null || Integer.parseInt(length.toString()) == 0
                     || status >= 400) {
-                    return cls == Response.class ? r : cls == InputStream.class ? inputStream : null;
+                    return cls == Response.class ? r : status >= 400 ? inputStream : null;
                 }
             }
         } catch (IOException ex) {
@@ -400,13 +445,12 @@ public class AbstractClient implements Client {
                 return mbr.readFrom(cls, type, anns, contentType, 
                        new MetadataMap<String, Object>(r.getMetadata(), true, true), inputStream);
             } catch (Exception ex) {
-                throw new WebApplicationException(ex);
+                reportMessageHandlerProblem("MSG_READER_PROBLEM", cls, contentType, ex, r);
             }
-             
         } else if (cls == Response.class) {
             return r;
         } else {
-            reportNoMessageHandler("NO_MSG_READER", cls, contentType);
+            reportMessageHandlerProblem("NO_MSG_READER", cls, contentType, null, null);
         }
         return null;                                                
     }
@@ -419,7 +463,18 @@ public class AbstractClient implements Client {
                                                + "with matrix and query parameters only");
         }
         if (!"".equals(paramName)) {
-            addToBuilder(ub, paramName, pValue, pt);    
+            
+            if (InjectionUtils.isSupportedCollectionOrArray(pValue.getClass())) {
+                Collection<?> c = pValue.getClass().isArray() 
+                    ? Arrays.asList((Object[]) pValue) : (Collection) pValue;
+                for (Iterator<?> it = c.iterator(); it.hasNext();) {
+                    addToBuilder(ub, paramName, it.next(), pt);
+                }
+            } else { 
+                addToBuilder(ub, paramName, pValue, pt); 
+            }
+                
+                    
         } else {
             MultivaluedMap<String, Object> values = 
                 InjectionUtils.extractValuesFromBean(pValue, "");
@@ -440,14 +495,15 @@ public class AbstractClient implements Client {
         }
     }
     
-    protected static void reportNoMessageHandler(String name, Class<?> cls, MediaType ct) {
+    protected static void reportMessageHandlerProblem(String name, Class<?> cls, MediaType ct, 
+                                                      Throwable cause, Response response) {
         org.apache.cxf.common.i18n.Message errorMsg = 
             new org.apache.cxf.common.i18n.Message(name, 
                                                    BUNDLE,
                                                    cls,
                                                    ct.toString());
         LOG.severe(errorMsg.toString());
-        throw new WebApplicationException(415);
+        throw new ClientWebApplicationException(errorMsg.toString(), cause, response);
     }
     
     private static MediaType getResponseContentType(Response r) {
@@ -466,7 +522,7 @@ public class AbstractClient implements Client {
             connect.setRequestMethod(methodName);
             return connect;
         } catch (Exception ex) {
-            throw new WebApplicationException(ex);
+            throw new ClientWebApplicationException("REMOTE_CONNECTION_PROBLEM", ex, null);
         }
     }
     
@@ -516,17 +572,17 @@ public class AbstractClient implements Client {
     
     protected static PhaseInterceptorChain setupOutInterceptorChain(ClientConfiguration cfg) { 
         PhaseManager pm = cfg.getBus().getExtension(PhaseManager.class);
-        List<Interceptor> i1 = cfg.getBus().getOutInterceptors();
-        List<Interceptor> i2 = cfg.getOutInterceptors();
-        List<Interceptor> i3 = cfg.getConduitSelector().getEndpoint().getOutInterceptors();
+        List<Interceptor<? extends Message>> i1 = cfg.getBus().getOutInterceptors();
+        List<Interceptor<? extends Message>> i2 = cfg.getOutInterceptors();
+        List<Interceptor<? extends Message>> i3 = cfg.getConduitSelector().getEndpoint().getOutInterceptors();
         return new PhaseChainCache().get(pm.getOutPhases(), i1, i2, i3);
     }
     
     protected static PhaseInterceptorChain setupInInterceptorChain(ClientConfiguration cfg) { 
         PhaseManager pm = cfg.getBus().getExtension(PhaseManager.class);
-        List<Interceptor> i1 = cfg.getBus().getInInterceptors();
-        List<Interceptor> i2 = cfg.getInInterceptors();
-        List<Interceptor> i3 = cfg.getConduitSelector().getEndpoint().getInInterceptors();
+        List<Interceptor<? extends Message>> i1 = cfg.getBus().getInInterceptors();
+        List<Interceptor<? extends Message>> i2 = cfg.getInInterceptors();
+        List<Interceptor<? extends Message>> i3 = cfg.getConduitSelector().getEndpoint().getInInterceptors();
         
         return new PhaseChainCache().get(pm.getInPhases(), i1, i2, i3);
     }
@@ -558,6 +614,9 @@ public class AbstractClient implements Client {
         exchange.put(MessageObserver.class, new ClientMessageObserver(cfg));
         exchange.put(Endpoint.class, cfg.getConduitSelector().getEndpoint());
         exchange.setOneWay("true".equals(headers.getFirst(Message.ONE_WAY_REQUEST)));
+        // no need for the underlying conduit to throw the IO exceptions in case of
+        // client requests returning error HTTP code, it can be overridden if really needed 
+        exchange.put("org.apache.cxf.http.no_io_exceptions", true);
         m.setExchange(exchange);
         
         PhaseInterceptorChain chain = setupOutInterceptorChain(cfg);
@@ -571,6 +630,7 @@ public class AbstractClient implements Client {
             m.put(Message.INVOCATION_CONTEXT, context);
             m.putAll(cfg.getRequestContext());
             exchange.putAll(cfg.getRequestContext());
+            exchange.putAll(cfg.getResponseContext());
         }
         
         //setup conduit selector
