@@ -33,8 +33,8 @@ public class WebClientTest extends Assert {
     @Test
     public void testEncoding() {
         URI u = WebClient.create("http://foo").path("bar+ %2B").matrix("a", "value+ ")
-            .query("b", "bv+ ").getCurrentURI();
-        assertEquals("http://foo/bar+%20%2B;a=value+%20?b=bv%2B+", u.toString());
+            .query("b", "bv+ %2B").getCurrentURI();
+        assertEquals("http://foo/bar+%20%2B;a=value+%20?b=bv%2B+%2B", u.toString());
     }
     
     @Test
