@@ -191,15 +191,30 @@ public class URITemplateTest extends Assert {
 
     @Test
     public void testBasicCustomExpression() throws Exception {
-        URITemplate uriTemplate = new URITemplate("/books/{bookId:[^/]+?}");
+        doTestBasicCustomExpression("/books/{bookId:[^/]+?}");
+    }
+    
+    @Test
+    public void testBasicCustomExpressionWithSpaces() throws Exception {
+        doTestBasicCustomExpression("/books/{ bookId : [^/]+? }");
+    }
+    
+    @Test
+    public void testBasicCustomExpressionWithSpaces2() throws Exception {
+        doTestBasicCustomExpression("/books/{ bookId }/");
+    }
+    
+    private void doTestBasicCustomExpression(String expression) {
+        URITemplate uriTemplate = new URITemplate(expression);
         MultivaluedMap<String, String> values = new MetadataMap<String, String>();
 
         boolean match = uriTemplate.match("/books/123/chapter/1", values);
         assertTrue(match);
         assertEquals("123", values.getFirst("bookId"));
         String subResourcePath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
-        assertEquals("/chapter/1", subResourcePath);
+        assertEquals("/chapter/1", subResourcePath);    
     }
+    
 
     @Test
     public void testBasicCustomExpression2() throws Exception {
@@ -270,6 +285,58 @@ public class URITemplateTest extends Assert {
         assertFalse(uriTemplate.match("/script.pdf", values));
     }
 
+    @Test
+    public void testExpressionWithNestedGroupAndTwoVars() throws Exception {
+        URITemplate uriTemplate = new URITemplate("/foo/{bar}/{resource:.+\\.(js|css|gif|png)}");
+        MultivaluedMap<String, String> values = new MetadataMap<String, String>();
+
+        assertTrue(uriTemplate.match("/foo/1/script.js", values));
+        assertEquals("1", values.getFirst("bar"));
+        assertEquals("script.js", values.getFirst("resource"));
+        String finalPath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
+        assertEquals("/", finalPath);
+    }
+    
+    @Test
+    public void testExpressionWithNestedGroupAndTwoVars2() throws Exception {
+        URITemplate uriTemplate = new URITemplate("/foo/{bar}{resource:(/format/[^/]+?)?}");
+        MultivaluedMap<String, String> values = new MetadataMap<String, String>();
+
+        assertTrue(uriTemplate.match("/foo/1/format", values));
+        assertEquals("1", values.getFirst("bar"));
+        assertEquals("/format", values.getFirst("resource"));
+        String finalPath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
+        assertEquals("/", finalPath);
+        values.clear();
+        
+        assertTrue(uriTemplate.match("/foo/1/format/2", values));
+        assertEquals("1", values.getFirst("bar"));
+        assertEquals("/format/2", values.getFirst("resource"));
+        finalPath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
+        assertEquals("/", finalPath);
+        values.clear();
+        
+        assertTrue(uriTemplate.match("/foo/1", values));
+        assertEquals("1", values.getFirst("bar"));
+        assertNull(values.getFirst("resource"));
+        finalPath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
+        assertEquals("/", finalPath);
+    }
+    
+    
+    @Test
+    public void testExpressionWithNestedGroupAndManySegments() throws Exception {
+        URITemplate uriTemplate = new URITemplate("/foo/{bar}{resource:(/format/[^/]+?)?}/baz");
+        MultivaluedMap<String, String> values = new MetadataMap<String, String>();
+
+        assertTrue(uriTemplate.match("/foo/1/format/2/baz/3", values));
+        assertEquals("1", values.getFirst("bar"));
+        assertEquals("/format/2", values.getFirst("resource"));
+        String finalPath = values.getFirst(URITemplate.FINAL_MATCH_GROUP);
+        assertEquals("/3", finalPath);
+        values.clear();
+    }
+    
     @Test
     public void testExpressionWithNestedGroup2() throws Exception {
         URITemplate uriTemplate = 
