@@ -31,6 +31,7 @@ import javax.ws.rs.Produces;
 import javax.ws.rs.QueryParam;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.Response;
 import javax.xml.bind.annotation.XmlTransient;
 
 import org.apache.cxf.aegis.type.java5.IgnoreProperty;
@@ -102,6 +103,13 @@ public class BookStore {
         return new Chapter(1);
     }
     
+    @GET
+    @Path("chapter2")
+    @WadlElement(response = Chapter.class)
+    public Response getChaper2() {
+        return Response.ok().entity(new Chapter(1)).build();
+    }
+    
     @Path("form")
     public FormInterface getForm() {
         return new Book(1);
@@ -137,6 +145,10 @@ public class BookStore {
             return bean;
         }
         
+        public TestEnum getE() {
+            return TestEnum.A;
+        }
+        
     }
     
     public static class QueryBean2 {
@@ -171,4 +183,8 @@ public class BookStore {
             return b;
         }
     }
+    
+    public static enum TestEnum {
+        A;        
+    }
 }
