@@ -46,16 +46,19 @@ import org.apache.cxf.phase.AbstractPhaseInterceptor;
 import org.apache.cxf.phase.Phase;
 import org.apache.cxf.service.Service;
 import org.apache.cxf.service.invoker.Invoker;
+import org.apache.cxf.service.model.BindingOperationInfo;
 import org.apache.cxf.staxutils.W3CDOMStreamWriter;
 import org.apache.cxf.transport.Destination;
 import org.apache.cxf.ws.addressing.AddressingProperties;
 import org.apache.cxf.ws.addressing.AddressingPropertiesImpl;
 import org.apache.cxf.ws.addressing.AttributedURIType;
 import org.apache.cxf.ws.addressing.JAXWSAConstants;
+import org.apache.cxf.ws.addressing.MAPAggregator;
 import org.apache.cxf.ws.policy.AssertionInfo;
 import org.apache.cxf.ws.policy.AssertionInfoMap;
 import org.apache.cxf.ws.policy.EndpointPolicy;
 import org.apache.cxf.ws.policy.PolicyAssertion;
+import org.apache.cxf.ws.policy.PolicyBuilder;
 import org.apache.cxf.ws.policy.PolicyEngine;
 import org.apache.cxf.ws.security.SecurityConstants;
 import org.apache.cxf.ws.security.policy.SP12Constants;
@@ -158,9 +161,11 @@ class SecureConversationInInterceptor extends AbstractPhaseInterceptor<SoapMessa
                         .getAddressingPolicy(aim, false);
                     all.addPolicyComponent(ass);
                     ea.addPolicyComponent(all);
-                    SymmetricBinding binding = new SymmetricBinding(SP12Constants.INSTANCE);
+                    PolicyBuilder pbuilder = message.getExchange().getBus()
+                        .getExtension(PolicyBuilder.class);
+                    SymmetricBinding binding = new SymmetricBinding(SP12Constants.INSTANCE, pbuilder);
                     binding.setIncludeTimestamp(true);
-                    ProtectionToken token = new ProtectionToken(SP12Constants.INSTANCE);
+                    ProtectionToken token = new ProtectionToken(SP12Constants.INSTANCE, pbuilder);
                     token.setToken(new SecureConversationToken(SP12Constants.INSTANCE));
                     binding.setProtectionToken(token);
                     binding.setEntireHeadersAndBodySignatures(true);
@@ -237,8 +242,8 @@ class SecureConversationInInterceptor extends AbstractPhaseInterceptor<SoapMessa
             message.getExchange().put(TokenStore.class.getName(), store);
         
             EndpointPolicy ep = pe.getServerEndpointPolicy(endpoint.getEndpointInfo(), destination);
-            List<Interceptor> interceptors = ep.getInterceptors();
-            for (Interceptor i : interceptors) {
+            List<Interceptor<? extends Message>> interceptors = ep.getInterceptors();
+            for (Interceptor<? extends Message> i : interceptors) {
                 message.getInterceptorChain().add(i);
             }
             
@@ -249,6 +254,9 @@ class SecureConversationInInterceptor extends AbstractPhaseInterceptor<SoapMessa
             endpoint.getService().setInvoker(new STSInvoker());
             ex.put(Endpoint.class, endpoint);
             ex.put(Service.class, endpoint.getService());
+            ex.put(org.apache.cxf.binding.Binding.class, endpoint.getBinding());
+            ex.remove(BindingOperationInfo.class);
+            message.put(MAPAggregator.ACTION_VERIFIED, Boolean.TRUE);
         } catch (Exception exc) {
             throw new Fault(exc);
         }
@@ -450,22 +458,24 @@ class SecureConversationInInterceptor extends AbstractPhaseInterceptor<SoapMessa
             //Find the SC token
             boolean found = false;
             List results = (List)message.get(WSHandlerConstants.RECV_RESULTS);
-            for (int i = 0; i < results.size(); i++) {
-                WSHandlerResult rResult =
-                        (WSHandlerResult) results.get(i);
-
-                Vector wsSecEngineResults = rResult.getResults();
-
-                for (int j = 0; j < wsSecEngineResults.size(); j++) {
-                    WSSecurityEngineResult wser =
-                            (WSSecurityEngineResult) wsSecEngineResults.get(j);
-                    Integer actInt = (Integer)wser.get(WSSecurityEngineResult.TAG_ACTION);
-                    if (actInt.intValue() == WSConstants.SCT) {
-                        SecurityContextToken tok
-                            = (SecurityContextToken)wser
-                                .get(WSSecurityEngineResult.TAG_SECURITY_CONTEXT_TOKEN);
-                        message.getExchange().put(SecurityConstants.TOKEN_ID, tok.getIdentifier());
-                        found = true;
+            if (results != null) {
+                for (int i = 0; i < results.size(); i++) {
+                    WSHandlerResult rResult =
+                            (WSHandlerResult) results.get(i);
+    
+                    Vector wsSecEngineResults = rResult.getResults();
+    
+                    for (int j = 0; j < wsSecEngineResults.size(); j++) {
+                        WSSecurityEngineResult wser =
+                                (WSSecurityEngineResult) wsSecEngineResults.get(j);
+                        Integer actInt = (Integer)wser.get(WSSecurityEngineResult.TAG_ACTION);
+                        if (actInt.intValue() == WSConstants.SCT) {
+                            SecurityContextToken tok
+                                = (SecurityContextToken)wser
+                                    .get(WSSecurityEngineResult.TAG_SECURITY_CONTEXT_TOKEN);
+                            message.getExchange().put(SecurityConstants.TOKEN_ID, tok.getIdentifier());
+                            found = true;
+                        }
                     }
                 }
             }
