@@ -26,8 +26,11 @@ import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
+import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.Response;
 
 import org.apache.commons.httpclient.Header;
@@ -42,8 +45,10 @@ import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactoryBean;
+import org.apache.cxf.jaxrs.client.ServerWebApplicationException;
 import org.apache.cxf.jaxrs.client.WebClient;
 import org.apache.cxf.jaxrs.ext.xml.XMLSource;
+import org.apache.cxf.jaxrs.provider.JAXBElementProvider;
 import org.apache.cxf.jaxrs.provider.XSLTJaxbProvider;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 
@@ -57,7 +62,21 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     @BeforeClass
     public static void startServers() throws Exception {
         assertTrue("server did not launch correctly",
-                   launchServer(BookServer.class, true));
+                   launchServer(BookServer.class));
+    }
+    
+
+    @Test
+    public void testGetCollectionOfBooks() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/collections"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        wc.accept("application/xml");
+        Collection<? extends Book> collection = wc.getCollection(Book.class);
+        assertEquals(1, collection.size());
+        Book book = collection.iterator().next();
+        assertEquals(123L, book.getId());
     }
     
     
@@ -68,6 +87,40 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(202, r.getStatus());
     }
     
+    @Test
+    public void testTempRedirectWebClient() throws Exception {
+        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/tempredirect");
+        Response r = client.type("*/*").get();
+        assertEquals(307, r.getStatus());
+        MultivaluedMap<String, Object> map = r.getMetadata();
+        assertEquals("http://localhost:" + PORT + "/whatever/redirection?css1=http%3A//bar",
+                     map.getFirst("Location").toString());
+        List<Object> cookies = r.getMetadata().get("Set-Cookie");
+        assertNotNull(cookies);
+        assertEquals(2, cookies.size());
+    }
+    
+    @Test
+    public void testSetCookieWebClient() throws Exception {
+        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/setcookies");
+        Response r = client.type("*/*").get();
+        assertEquals(200, r.getStatus());
+        List<Object> cookies = r.getMetadata().get("Set-Cookie");
+        assertNotNull(cookies);
+        assertEquals(1, cookies.size());
+    }
+    
+    @Test
+    public void testSetTwoCookieWebClient() throws Exception {
+        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/settwocookies");
+        Response r = client.type("*/*").get();
+        assertEquals(200, r.getStatus());
+        List<Object> cookies = r.getMetadata().get("Set-Cookie");
+        assertNotNull(cookies);
+        assertEquals(2, cookies.size());
+    }
+
+    
     @Test
     public void testOnewayProxy() throws Exception {
         BookStore proxy = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
@@ -75,6 +128,16 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(202, WebClient.client(proxy).getResponse().getStatus());
     }
     
+    @Test
+    public void testProxyWithCollectionMatrixParams() throws Exception {
+        BookStore proxy = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        List<String> params = new ArrayList<String>(); 
+        params.add("12");
+        params.add("3");
+        Book book = proxy.getBookByMatrixListParams(params);
+        assertEquals(123L, book.getId());
+    }
+    
     @Test
     public void testPropogateException() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/propogateexception",
@@ -105,6 +168,33 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       data, "application/xml", 500);
     }
     
+    @Test
+    public void testServerWebApplicationException() throws Exception {
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/webappexception");
+        wc.accept("application/xml");
+        try {
+            wc.get(Book.class);
+            fail("Exception expected");
+        } catch (ServerWebApplicationException ex) {
+            assertEquals(500, ex.getStatus());
+            assertEquals("This is a WebApplicationException", ex.getMessage());
+            assertTrue(ex.toString().contains("This is a WebApplicationException"));
+        }
+    }
+    
+    @Test
+    public void testServerWebApplicationExceptionWithProxy() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        try {
+            store.throwException();
+            fail("Exception expected");
+        } catch (ServerWebApplicationException ex) {
+            assertEquals(500, ex.getStatus());
+            assertEquals("This is a WebApplicationException", ex.getMessage());
+            assertTrue(ex.toString().contains("This is a WebApplicationException"));
+        }
+    }
+    
     @Test
     public void testWebApplicationException() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/webappexception",
@@ -223,6 +313,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertTrue(values.contains("POST") && values.contains("GET")
                    && values.contains("DELETE") && values.contains("PUT"));
         assertEquals(0, ((InputStream)response.getEntity()).available());
+        List<Object> date = response.getMetadata().get("Date");
+        assertNotNull(date);
+        assertEquals(1, date.size());
     }
     
     @Test
@@ -487,6 +580,23 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testSearchBook123() throws Exception {
+        getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/search"
+                               + "?_s=name==CXF*;id=ge=123;id=lt=124",
+                               "resources/expected_get_book123.txt",
+                               "application/xml", 200);
+    }
+    
+    @Test
+    public void testSearchBook123WithWebClient() throws Exception {
+        String address = "http://localhost:" + PORT + "/bookstore/books/search";
+                          
+        WebClient client = WebClient.create(address);
+        Book b = client.query("_s", "name==CXF*;id=ge=123;id=lt=124").get(Book.class);
+        assertEquals(b.getId(), 123L);
+        
+    }
     
     @Test
     public void testGetBook123() throws Exception {
@@ -597,6 +707,22 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testPostGetBookAdapterList() throws Exception {
+        JAXBElementProvider provider = new JAXBElementProvider();
+        Map<String, String> outMap = new HashMap<String, String>();
+        outMap.put("Books", "CollectionWrapper");
+        outMap.put("books", "Book");
+        provider.setOutTransformElements(outMap);
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/books/adapter-list",
+                                        Collections.singletonList(provider));
+        
+        Collection<? extends Book> books = wc.type("application/xml").accept("application/xml")
+            .postAndGetCollection(new Books(new Book("CXF", 123L)), Book.class);
+        assertEquals(1, books.size());
+        assertEquals(123L, books.iterator().next().getId());
+    }
+    
     @Test
     public void testGetBookAdapterInterface() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/interface/adapter",
@@ -734,7 +860,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(888L, book.getId());
         Response r = client.getResponse();
         assertEquals("CustomValue", r.getMetadata().getFirst("CustomHeader"));
-        assertEquals(333, r.getStatus());
+        assertEquals(233, r.getStatus());
         assertEquals("application/xml", r.getMetadata().getFirst("Content-Type"));
     }
     
