@@ -37,6 +37,8 @@ import javax.ws.rs.core.Response;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.Unmarshaller;
 import javax.xml.namespace.QName;
+import javax.xml.ws.Service;
+import javax.xml.ws.soap.SOAPBinding;
 
 import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.methods.FileRequestEntity;
@@ -61,6 +63,11 @@ import org.apache.cxf.jaxrs.provider.JAXBElementProvider;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.phase.AbstractPhaseInterceptor;
 import org.apache.cxf.phase.Phase;
+import org.apache.cxf.systest.jaxrs.jaxws.BookSoapService;
+import org.apache.cxf.systest.jaxrs.jaxws.BookStoreJaxrsJaxws;
+import org.apache.cxf.systest.jaxrs.jaxws.HelloWorld;
+import org.apache.cxf.systest.jaxrs.jaxws.User;
+import org.apache.cxf.systest.jaxrs.jaxws.UserImpl;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 import org.apache.cxf.transport.http.HTTPConduit;
 
@@ -74,7 +81,42 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
     @BeforeClass
     public static void startServers() throws Exception {
         assertTrue("server did not launch correctly", 
-                   launchServer(BookServerRestSoap.class, true));
+                   launchServer(BookServerRestSoap.class));
+    }
+    
+    @Test
+    public void testHelloRest() throws Exception {
+        String address = "http://localhost:" + PORT + "/test/services/hello-rest";
+        
+        HelloWorld service = JAXRSClientFactory.create(address, HelloWorld.class);
+        useHelloService(service);
+    }
+    
+    @Test
+    public void testHelloSoap() throws Exception {
+        final QName serviceName = new QName("http://hello.com", "HelloWorld");
+        final QName portName = new QName("http://hello.com", "HelloWorldPort");
+        final String address = "http://localhost:" + PORT + "/test/services/hello-soap";
+        
+        Service service = Service.create(serviceName);
+        service.addPort(portName, SOAPBinding.SOAP11HTTP_BINDING, address);
+    
+        HelloWorld hw = service.getPort(HelloWorld.class); 
+    
+        useHelloService(hw);
+    }
+    
+    private void useHelloService(HelloWorld service) {
+        assertEquals("Hello Barry", service.sayHi("Barry"));
+        assertEquals("Hello Fred", service.sayHiToUser(new UserImpl("Fred")));
+        
+        Map<Integer, User> users = service.getUsers();
+        assertEquals(1, users.size());
+        assertEquals("Fred", users.entrySet().iterator().next().getValue().getName());
+        
+        users = service.echoUsers(users);
+        assertEquals(1, users.size());
+        assertEquals("Fred", users.entrySet().iterator().next().getValue().getName());
     }
     
     @Test
@@ -543,7 +585,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
     @Test
     public void testServerFaultInInterceptor() throws Exception {
         //testing faults created by server handled correctly
-        serverFaultInInterceptorTest("321");
+        
         //999 causes error code of 404, 404 has a different code path so need to test too
         serverFaultInInterceptorTest("999");
         //322 causes a checked exception to be thrown so need to 
@@ -586,6 +628,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         features.add((AbstractFeature)testFeature);
         bean.setFeatures(features);
         BookStoreJaxrsJaxws proxy = (BookStoreJaxrsJaxws)bean.create();
+        WebClient.getConfig(proxy).getRequestContext().put("org.apache.cxf.http.no_io_exceptions", false);
         try {
             //321 is special case - causes error code of 525
             proxy.getBook(new Long(param));
