@@ -702,7 +702,7 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
                 BindingProvider bp = (BindingProvider)greeter;
                 Map<String, Object> responseContext = bp.getResponseContext();
                 String contentType = (String) responseContext.get(Message.CONTENT_TYPE);
-                assertEquals("text/xml; charset=utf-8", contentType);
+                assertEquals("text/xml;charset=utf-8", contentType.toLowerCase());
                 Integer responseCode = (Integer) responseContext.get(Message.RESPONSE_CODE);
                 assertEquals(500, responseCode.intValue());                
                 assertNotNull(brlf.getFaultInfo());
@@ -745,7 +745,7 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
         
         assertEquals(200, httpConnection.getResponseCode());
         
-        assertEquals("text/xml; charset=utf-8", httpConnection.getContentType());
+        assertEquals("text/xml;charset=utf-8", httpConnection.getContentType().toLowerCase());
         assertEquals("OK", httpConnection.getResponseMessage());
         
         InputStream in = httpConnection.getInputStream();
@@ -775,7 +775,7 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
         
         assertEquals(200, httpConnection.getResponseCode());
     
-        assertEquals("text/xml; charset=utf-8", httpConnection.getContentType());
+        assertEquals("text/xml;charset=utf-8", httpConnection.getContentType().toLowerCase());
         assertEquals("OK", httpConnection.getResponseMessage());
         
         InputStream in = httpConnection.getInputStream();
@@ -826,7 +826,7 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
         
         assertEquals(200, httpConnection.getResponseCode());
     
-        assertEquals("text/xml; charset=utf-8", httpConnection.getContentType());
+        assertEquals("text/xml;charset=utf-8", httpConnection.getContentType().toLowerCase());
         assertEquals("OK", httpConnection.getResponseMessage());
         
         InputStream in = httpConnection.getInputStream();
@@ -1003,4 +1003,21 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
         ins.close();
     }
     
+    @Test
+    public void testGetPortWithoutPortName() throws Exception {
+        SOAPService service = new SOAPService();
+        assertNotNull(service);
+        Greeter greeter = service.getPort(Greeter.class);
+        updateAddressPort(greeter, PORT);
+        try {
+            greeter.greetMe("test");           
+            String reply = greeter.sayHi();
+            assertNotNull("no response received from service", reply);
+            assertEquals("Bonjour", reply);
+        } catch (UndeclaredThrowableException ex) {
+            throw (Exception)ex.getCause();
+        }
+    }
+    
+    
 }
