@@ -1136,4 +1136,31 @@ public class CodeGenBugTest extends AbstractCodeGenTest {
                             "org/apache/cxf/w2j/hello_world_soap_http/TestServiceName1.java").exists());
     }
 
+    
+    @Test
+    public void testCXF3105() throws Exception {
+        String[] args = new String[] {"-d", output.getCanonicalPath(),
+            "-impl", "-server", "-client", 
+            "-b", getLocation("/wsdl2java_wsdl/cxf3105/ws-binding.xml"),
+            getLocation("/wsdl2java_wsdl/cxf3105/cxf3105.wsdl")};
+        WSDLToJava.main(args);
+
+        assertNotNull(output);
+        File f = new File(output, "org/apache/cxf/testcase/cxf3105/Login.java");
+        assertTrue(f.exists());
+        String contents = IOUtils.readStringFromStream(new FileInputStream(f));
+        assertTrue(contents.contains("Loginrequesttype loginRequest"));
+        assertTrue(contents.contains("<Loginresponsetype> loginResponse"));
+    }
+
+    @Test
+    public void testOverloadWithAction() throws Exception {
+        String[] args = new String[] {"-d", output.getCanonicalPath(),
+            getLocation("/wsdl2java_wsdl/hello_world_overload.wsdl")};
+        WSDLToJava.main(args);
+        
+        assertNotNull(output);
+        File f = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/SayHi.java");
+        assertTrue(f.exists());
+    }
 }
